// ===========================================================================
// CONTENT  : TEST CLASS SecurityUtilTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 18/08/2014
// HISTORY  :
//  18/08/2014  mdu  CREATED
//
// Copyright (c) 2014, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.security;

import static org.junit.Assert.*;
import static org.pfsw.security.testhelper.UnitTestHelper.*;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;

import org.junit.Test;
import org.pfsw.bif.text.IStringPair;
import org.pfsw.security.x509.GeneralName;
import org.pfsw.text.CheckedCharsetName;

public class SecurityUtilTest
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final SecurityUtil SECU = SecurityUtil.current();

  // =========================================================================
  // TEST METHODS
  // =========================================================================
  @Test
  public void test_encodeBASICCredentials()
  {
    assertEquals("UGV0ZXJQYW46Tm90SG9va2Vk", SECU.encodeBASICCredentials("PeterPan", "NotHooked"));
  }

  @Test
  public void test_encodeBASICCredentials_ISO_8859_1()
  {
    assertEquals("1npn/Gw6P1NwYd8=", SECU.encodeBASICCredentials("Özgül", "€Spaß", Charset.forName("ISO-8859-1")));
  }

  @Test
  public void test_decodeBASICCredentials_ISO_8859_1()
  {
    String encoded = "QvzfZXI6QvZH3Gy14vM==";
    IStringPair pair;

    pair = SECU.decodeBASICCredentials(encoded, Charset.forName("ISO-8859-1"));
    assertEquals("Büßer", pair.getString1());
    assertEquals("BöGÜlµâó", pair.getString2());
  }

  @Test(expected = IllegalArgumentException.class)
  public void test_decodeBASICCredentials__missing_colon()
  {
    SECU.decodeBASICCredentials("cmFzcHV0aW4=", CheckedCharsetName.UTF_8.getCharset());
  }

  @Test(expected = CertificateException.class)
  public void test_readX509CertificateFromPKCS7__file_not_found() throws CertificateException
  {
    SECU.readX509CertificateFromPKCS7(new File("unknown.pb7"));
  }

  @Test
  public void test_decodeBASICCredentials()
  {
    String encoded = "TWlja2V5TW91c2U6TWFybWVsYWRlMTY0NDY=";
    IStringPair pair;

    pair = SECU.decodeBASICCredentials(encoded);
    assertEquals("MickeyMouse", pair.getString1());
    assertEquals("Marmelade16446", pair.getString2());
  }

  @Test
  public void test_decodeBASICCredentials_not_splitable()
  {
    try
    {
      SECU.decodeBASICCredentials("UnViYmVyYm9hdDI1==");
      fail("Expected IllegalArgumentException");
    }
    catch (Exception ex)
    {
      assertTrue(ex instanceof IllegalArgumentException);
      assertTrue(ex.getMessage().contains("split"));
    }
  }

  @Test
  public void test_decodeBASICCredentials_not_splittable()
  {
    try
    {
      SECU.decodeBASICCredentials("OjdSdWJiZXJib2F0MjU=");
      fail("Expected IllegalArgumentException");
    }
    catch (Exception ex)
    {
      assertTrue(ex instanceof IllegalArgumentException);
      assertTrue(ex.getMessage().contains("split"));
    }
  }

  @Test
  public void test_createBASICAuthorization()
  {
    String authorization;

    authorization = SECU.createBASICAuthorization("Paul", "VerySecret99");
    assertEquals("Basic UGF1bDpWZXJ5U2VjcmV0OTk=", authorization);
  }

  @Test
  public void test_createBEARERAuthorization() throws UnsupportedEncodingException
  {
    String authorization;

    authorization = SECU.createBEARERAuthorization("s/Z&§=}~H2#'?³(/&=S|AüSD@J7€zßdjdj".getBytes("UTF-8"));
    assertEquals("Bearer cy9aJsKnPX1+SDIjJz/CsygvJj1TfEHDvFNEQEo34oKsesOfZGpkag==", authorization);
  }

  @Test
  public void test_createBEARERAuthorization_ISO_8859_1()
  {
    String authorization;

    authorization = SECU.createBEARERAuthorization("ÜüÖDÄÖLDOIW?ßjdkynjxgjn", Charset.forName("ISO-8859-1"));
    assertEquals("Bearer 3PzWRMTWTERPSVc/32pka3luanhnam4=", authorization);
  }

  @Test
  public void test_readX509CertificateChainFromPEM_github()
  {
    X509Certificate[] chain;
    List<GeneralName> names;

    chain = SECU.readX509CertificateChainFromPEM(testDataFile("github-com-chain.pem"));
    assertEquals(3, chain.length);
    names = SECU.getSubjectAlternativeNamesFrom(chain[0]);
    assertEquals("github.com", names.get(0).asString());
    assertEquals("www.github.com", names.get(1).asString());
  }
}
