// ===========================================================================
// CONTENT  : TEST CLASS AutoAuthenticationManagerTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 12/09/2004
// HISTORY  :
//  12/09/2004  duma  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.security.authentication;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;

import java.net.Authenticator;
import java.net.InetAddress;
import java.net.PasswordAuthentication;
import java.net.UnknownHostException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class AutoAuthenticationManagerTest
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  protected static final String HOST_1 = "www.pf.org";
  protected static final String BASIC = "Basic";
  protected static final String REALM_1 = "Test-Realm 1";
  protected static final String REALM_2 = "Test-Realm 2";
  protected static final String REALM_3 = "Test-Realm 3";
  protected static final String REALM_4 = "Test-Realm 4";
  protected static final String REALM_5 = "Test-Realm 5";

  protected static InetAddress IP_ADDRESS_1 = null;

  static
  {
    try
    {
      IP_ADDRESS_1 = InetAddress.getByName("122.218.33.17");
    }
    catch (UnknownHostException e)
    {
      e.printStackTrace();
    }
  }

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  protected AutoAuthenticationManager aam = AutoAuthenticationManager.instance();

  // =========================================================================
  // TEST METHODS
  // =========================================================================

  @Test
  public void test_install_1() throws Exception
  {
    PasswordAuthentication auth;
    LocationCredentials creds;

    creds = new LocationCredentials(REALM_1, "Freddie", "Mercury");
    aam.register("R1", creds);

    auth = this.getAuthCredentials();
    assertEquals("Freddie", auth.getUserName());
    assertEquals("Mercury", new String(auth.getPassword()));
  }

  @Test
  public void test_authByRealm_1() throws Exception
  {
    PasswordAuthentication auth;
    LocationCredentials creds;

    creds = new LocationCredentials(REALM_1, "Freddie", "Mercury");
    aam.register("R1", creds);
    creds = new LocationCredentials(REALM_2, "Brian", "May");
    aam.register("R2", creds);
    creds = new LocationCredentials(REALM_3, "John", "Deacon");
    aam.register("R3", creds);

    auth = this.getAuthCredentials(REALM_1);
    assertEquals("Freddie", auth.getUserName());
    assertEquals("Mercury", new String(auth.getPassword()));
    auth = this.getAuthCredentials(REALM_2);
    assertEquals("Brian", auth.getUserName());
    assertEquals("May", new String(auth.getPassword()));
    auth = this.getAuthCredentials(REALM_3);
    assertEquals("John", auth.getUserName());
    assertEquals("Deacon", new String(auth.getPassword()));
    auth = this.getAuthCredentials(REALM_4);
    assertNull(null);
  }

  @Test
  public void test_authByURL_1() throws Exception
  {
    PasswordAuthentication auth;
    LocationCredentials creds;

    creds = new LocationCredentials(REALM_1, "Freddie", "Mercury");
    creds.addPattern("http://www.test.com/search/*");
    aam.register("R1", creds);
    creds = new LocationCredentials(REALM_2, "Brian", "May");
    creds.addPattern("http://www.test.com/order/*");
    aam.register("R2", creds);
    creds = new LocationCredentials(REALM_3, "John", "Deacon");
    creds.addPattern("http://www.protected.com/webapp*/data*");
    aam.register("R3", creds);

    auth = this.getAuthCredentials(null);
    assertNull(null);

    aam.aboutToAccess("http://www.test.com/order/cars/bmw/z5");
    auth = this.getAuthCredentials(null);
    assertEquals("Brian", auth.getUserName());
    assertEquals("May", new String(auth.getPassword()));
  }

  @Test
  public void test_fillFallbackAuthenticator_1() throws Exception
  {
    PasswordAuthentication auth;
    AbstractAuthenticator fallback;

    fallback = new AbstractAuthenticator()
    {
      // do nothing 
    };
    aam.setFallbackAuthenticator(fallback);

    auth = this.getAuthCredentials(REALM_1);
    assertNull(auth);
    if (aam.supportsHostNames())
    {
      assertEquals(HOST_1, fallback.getRequestingHost());
    }
    else
    {
      assertNull(fallback.getRequestingHost());
    }
    assertEquals(80, fallback.getRequestingPort());
    assertEquals(REALM_1, fallback.getRequestingPrompt());
    assertEquals(BASIC, fallback.getRequestingScheme());
    assertEquals(IP_ADDRESS_1, fallback.getRequestingSite());
  }

  // =========================================================================
  // HELPER METHODS
  // =========================================================================
  @Before
  public void setUp()
  {
    AutoAuthenticationManager.install();
    aam.reset();
  }

  @After
  public void tearDown()
  {
    AutoAuthenticationManager.uninstall();
  }

  protected PasswordAuthentication getAuthCredentials()
  {
    return this.getAuthCredentials(REALM_1);
  }

  protected PasswordAuthentication getAuthCredentials(String realm)
  {
    if (aam.supportsHostNames())
    {
      return Authenticator.requestPasswordAuthentication(HOST_1, IP_ADDRESS_1, 80, "http", realm, BASIC);
    }
    return Authenticator.requestPasswordAuthentication(IP_ADDRESS_1, 80, "http", realm, BASIC);
  }

}
