// ===========================================================================
// CONTENT  : TEST CLASS CheckedHashAlgorithmTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 23/11/2013
// HISTORY  :
//  23/11/2013  mdu  CREATED
//
// Copyright (c) 2013, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.security.crypto;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;

import java.security.NoSuchAlgorithmException;

import org.junit.Test;
import org.pfsw.util.CollectionUtil;

/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow
 */
public class CheckedHashAlgorithmTest
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final CollectionUtil CU = CollectionUtil.current();

  // =========================================================================
  // TEST METHODS
  // =========================================================================
  @Test
  public void test_isSupportedAlgorithm_true()
  {
    assertTrue(CheckedHashAlgorithm.isSupportedAlgorithm("SHA-384"));
  }

  @Test
  public void test_isSupportedAlgorithm_false()
  {
    assertFalse(CheckedHashAlgorithm.isSupportedAlgorithm("UNKNOWN"));
  }

  @Test
  public void test_checkAlgorithm__null()
  {
    try
    {
      CheckedHashAlgorithm.checkAlgorithm(null);
      fail("Expected NoSuchAlgorithmException.");
    }
    catch (Exception ex)
    {
      assertTrue(ex instanceof NoSuchAlgorithmException);
    }
  }

  @Test
  public void test_create() throws NoSuchAlgorithmException
  {
    assertNotNull(CheckedHashAlgorithm.create("SHA-1"));
  }

  @Test
  public void test_getPrefix_MD5()
  {
    assertEquals(CryptoConstants.MD5_PREFIX, CheckedHashAlgorithm.MD5.getPrefix());
  }

  @Test
  public void test_getPrefix_SHA_1()
  {
    assertEquals(CryptoConstants.SHA_1_PREFIX, CheckedHashAlgorithm.SHA_1.getPrefix());
  }

  @Test
  public void test_getPrefix_SHA_256()
  {
    assertEquals(CryptoConstants.SHA_2_256_PREFIX, CheckedHashAlgorithm.SHA_2_256.getPrefix());
  }

  @Test
  public void test_getPrefix_SHA_384()
  {
    assertEquals(CryptoConstants.SHA_2_384_PREFIX, CheckedHashAlgorithm.SHA_2_384.getPrefix());
  }

  @Test
  public void test_getPrefix_SHA_512()
  {
    assertEquals(CryptoConstants.SHA_2_512_PREFIX, CheckedHashAlgorithm.SHA_2_512.getPrefix());
  }

  @Test
  public void test_getMessageDigest_SHA_1()
  {
    assertNotNull(CheckedHashAlgorithm.SHA_1.getMessageDigest());
  }

  @Test
  public void test_getMessageDigest_SHA_2_512()
  {
    assertNotNull(CheckedHashAlgorithm.SHA_2_512.getMessageDigest());
  }

  @Test
  public void test_getMessageDigest_MD5()
  {
    assertNotNull(CheckedHashAlgorithm.MD5.getMessageDigest());
  }

  @Test
  public void test_equals_and_hashCode() throws NoSuchAlgorithmException
  {
    CheckedHashAlgorithm alg1 = CheckedHashAlgorithm.MD5;
    CheckedHashAlgorithm alg2 = CheckedHashAlgorithm.create("md5");
    CheckedHashAlgorithm alg3 = CheckedHashAlgorithm.create("SHA-256");

    assertTrue(alg1 != alg2);
    assertTrue(alg1 != alg3);
    assertTrue(alg2 != alg3);
    assertTrue(alg1.equals(alg2));
    assertTrue(alg1.hashCode() == alg2.hashCode());
    assertFalse(alg1.equals(alg3));
    assertFalse(alg1.hashCode() == alg3.hashCode());
    assertFalse(alg1.equals(Integer.valueOf(17)));
  }

  @Test
  public void test_find_found()
  {
    assertSame(CheckedHashAlgorithm.SHA_2_384, CheckedHashAlgorithm.find("SHA-384"));
  }

  @Test
  public void test_find_not_found()
  {
    assertNull(CheckedHashAlgorithm.find("MD1"));
  }

  @Test
  public void test_getPredefinedAlgorithms()
  {
    CheckedHashAlgorithm[] algorithms;

    algorithms = CheckedHashAlgorithm.getPredefinedAlgorithms();
    assertTrue(algorithms.length >= 5);
    assertTrue(CU.contains(algorithms, CheckedHashAlgorithm.SHA_1));
  }

  @Test
  public void test_getBitLength()
  {
    assertEquals(128, CheckedHashAlgorithm.MD5.getBitLength());
    assertEquals(160, CheckedHashAlgorithm.SHA_1.getBitLength());
    assertEquals(256, CheckedHashAlgorithm.SHA_2_256.getBitLength());
    assertEquals(384, CheckedHashAlgorithm.SHA_2_384.getBitLength());
    assertEquals(512, CheckedHashAlgorithm.SHA_2_512.getBitLength());
  }

  @Test
  public void test_detectBitLength_invalid()
  {
    assertEquals(CheckedHashAlgorithm.UNKNOWN_LENGTH, CheckedHashAlgorithm.detectBitLength(null));
    assertEquals(CheckedHashAlgorithm.UNKNOWN_LENGTH, CheckedHashAlgorithm.detectBitLength("Alpha-Beta"));
    assertEquals(CheckedHashAlgorithm.UNKNOWN_LENGTH, CheckedHashAlgorithm.detectBitLength("UNKNOWN"));
  }

  @Test
  public void test_toString()
  {
    assertEquals("CheckedHashAlgorithm(SHA-256)", CheckedHashAlgorithm.SHA_2_256.toString());
  }
}
