// ===========================================================================
// CONTENT  : TEST CLASS {ClassName}
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 24/06/2008
// HISTORY  :
//  24/06/2008  mdu  CREATED
//
// Copyright (c) 2008, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.security.crypto;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Arrays;

import org.junit.Test;
import org.pfsw.security.SecurityUtil;
import org.pfsw.security.testhelper.UnitTestHelper;
import org.pfsw.text.CheckedCharsetName;
import org.pfsw.text.StringUtil;
import org.pfsw.util.Base64Converter;

/**
 * Test class for corresponding business class.
 *
 * @author M.Duchrow
 */
public class CryptoUtilTest
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final StringUtil SU = StringUtil.current();
  private static final SecurityUtil SECU = SecurityUtil.current();

  // =========================================================================
  // TEST METHODS
  // =========================================================================
  @Test
  public void test_asPlainText_1() throws Exception
  {
    assertNull(this.crypto().asPlainText(null));
  }

  @Test
  public void test_asPlainText_2() throws Exception
  {
    assertEquals("Jakarta", this.crypto().asPlainText("Jakarta"));
  }

  @Test
  public void test_asPlainText_3() throws Exception
  {
    assertEquals("{xyz}FromDuskTilDawn", this.crypto().asPlainText("{xyz}FromDuskTilDawn"));
  }

  @Test
  public void test_asPlainText_4() throws Exception
  {
    assertEquals("MissionImpossible", this.crypto().asPlainText("{XOR1}MgoFHBEIAyImDhEDIx8IAQkN"));
  }

  @Test
  public void test_asPlainText_5() throws Exception
  {
    assertEquals("G6fmBa$y", this.crypto().asPlainText("{XOR1}OQZCEh8rA1EN"));
  }

  @Test
  public void test_asPlainText_6() throws Exception
  {
    assertEquals("San{xor}Franzisco", this.crypto().asPlainText("San{xor}Franzisco"));
  }

  @Test
  public void test_asPlainText_7() throws Exception
  {
    assertEquals("The quick brown fox jumps over the lazy dog", this.crypto().asPlainText("{XOR1}MT0bM0EdHA0iH1QQGw0CGkUoDhVFKR0MAhJDGxMXG1MiCQlJCCAODVINDRI="));
  }

  @Test
  public void test_asPlainText_8() throws Exception
  {
    assertEquals("LynyrdSkynyrd", this.crypto().asPlainText("{XOR1}OQ0NGgsbBiYfHDwcEwA="));
  }

  @Test
  public void test_asPlainText_9() throws Exception
  {
    assertEquals("LynyrdSkynyrd", this.crypto().asPlainText("{XOR1}NwkXGxQXFjIfEAEXOxA="));
  }

  @Test
  public void test_xor1Encrypted_1() throws Exception
  {
    assertNull(this.crypto().xor1Encrypted(null));
  }

  @Test
  public void test_xor1Encrypted_2() throws Exception
  {
    String str = "Spandau Ballet";
    String encrypted;

    encrypted = this.crypto().xor1Encrypted(str);
    assertTrue(encrypted.startsWith("{XOR1}"));
    assertEquals(str, this.crypto().asPlainText(encrypted));
    //  	System.out.println( this.crypto().iap1Encrypted( "The quick brown fox jumps over the lazy dog" ) );
  }

  @Test
  public void test_equals_SSHA() throws Exception
  {
    assertTrue(this.crypto().equals("{SSHA}1sYykJ+MI4IMUkLmwhBxe1pdR25ST8hzlRrEVw==", "j2eesect12"));
  }

  @Test
  public void test_equals_XOR1() throws Exception
  {
    assertTrue(this.crypto().equals("{XOR1}ORMVHRwLDQJUQ3IyCQ0jFxoaBCoG", "{SSHA}bWC5Ln8mte8oFKJwxsoXVXCEWOcyMTUyNzM2MTA1MDUyNQ=="));
  }

  @Test
  public void test_equals_SSHA_b64() throws Exception
  {
    String hash = "e1NTSEF9WTR3RjNzdXBnNElQazFiNFJWY3VPUC9HSnNneU1UVXlOek0yTVRRd01qSXpOQT09";

    assertTrue(this.crypto().equals("Moonshadow", hash));
    assertFalse(this.crypto().equals("Moonslight", hash));
  }

  @Test
  public void test_equals_SHA_b64() throws Exception
  {
    assertTrue(this.crypto().equals("e1NIQX0vZzNQbXJjbjJmM1ZUYVFuZHBXZW13RGsxT0k9", "Stan & Ollie"));
  }

  @Test
  public void test_equals_XOR1_SHA_b64() throws Exception
  {
    //System.out.println( this.crypto().xorEncrypted( "10$Off#xR" ) );
    assertTrue(this.crypto().equals("10$Off#xR", "e1NIQX1oVUFrWjVNQVhndlBmeXlPTUZtWk5tRkxpNUU9"));
  }

  @Test
  public void test_equals_MD5() throws Exception
  {
    assertTrue(this.crypto().equals("Hulk Hogan", "{MD5}RiCoe13yMFFXcvB8bjab4w=="));
  }

  @Test
  public void test_equals_SHA_256() throws Exception
  {
    assertTrue(this.crypto().equals("No Sunshine today", "{SHA256}6vpJJ73n0bnhDYBbuz225ptMqFDB2wwvkcTuAHYT/R8="));
  }

  @Test
  public void test_equals_SAH_384() throws Exception
  {
    assertTrue(this.crypto().equals("Still got the blues", "{SHA384}jwKZGrzU9OdXgieSjB6LkrDKTLO2O2Rj9iq4bi5ZoJn3yy86dFAhjNKNz8KgVUp3"));
  }

  @Test
  public void test_equals_SHA_512() throws Exception
  {
    assertTrue(this.crypto().equals("{SHA512}G4K/29S0GICoMGF3xN0iROMWurlZCTOQeyxkVYe1qIav3NSMn6ClwhOV3eBCCkB2fCJOpjQDUZLir7PaYXFJIg==", "No Sunshine today"));
  }

  @Test
  public void test_equals_SSHA_256() throws Exception
  {
    assertTrue(this.crypto().equals("Öfter wär süß", "{SSHA256}lKv8GwCDbjB6wvYroiAKaGZXUdWYgtYAnpQWLTrWHPUSzuooRrOT5Yp70mbNwfx0IziQ+g=="));
  }

  @Test
  public void test_equals_SSHA_384_b64() throws Exception
  {
    String data = "Kochlöffel";
    String hash;

    hash = this.crypto().base64HashedUTF8(data, CryptoConstants.SSHA_2_384_NAME);
    assertTrue(this.crypto().equals(data, hash));
  }

  @Test
  public void test_equals_SSHA_512_b64() throws Exception
  {
    String data = "(§/%§(#'*~ÜvAußA?!äß\\^\"§ÖÄ|*ü'";
    String hash;

    hash = this.crypto().base64HashedUTF8(data, CryptoConstants.SSHA_2_512_NAME);
    assertTrue(this.crypto().equals(data, hash));
  }

  @Test
  public void test_equals_same()
  {
    // These two values are no hashes!
    assertTrue(this.crypto().equals("rKLSV7WM5ycov0FiH6qEbKSYRNehAEz+T2c58IFNAN0=", "rKLSV7WM5ycov0FiH6qEbKSYRNehAEz+T2c58IFNAN0="));
  }

  @Test
  public void test_equals_same_hashes()
  {
    String hash = "e1NTSEEyNTZ9azc5WFhGeVNpaFNHeXdpemxVWk9sNUY0Tmo0clQvRlpnNDJCeEtnQmF6SUV1d0tnNDd6RWp4blQvdmtEcFhyS01PSW55UT09";
    assertFalse(this.crypto().equals(hash, hash));
  }

  @Test
  public void test_equals_identical()
  {
    String string = "alpha";
    assertTrue(this.crypto().equals(string, string));
  }

  @Test
  public void test_equals_null_null()
  {
    assertFalse(this.crypto().equals(null, "test"));
    assertFalse(this.crypto().equals("test", null));
  }

  @Test
  public void test_equalsUTF8()
  {
    assertTrue(this.crypto().equalsUTF8("Öfter wär süß", "{SHA256}rKLSV7WM5ycov0FiH6qEbKSYRNehAEz+T2c58IFNAN0="));
    assertTrue(this.crypto().equalsUTF8("{SSHA512}7pLyRczt1l4wim1jYvW/CSqm3XmuqjobGU3NS7+YQWiK5qjwrP7i7v3lOvBIyNoTxPjnU4/D0Y4+BTMH8VhSMSOzuU3gKBNVxdc+newf0IHXkCQe", "Hast-la$Vista="));
  }

  @Test
  public void test_equals_SSHA512_against_XOR()
  {
    String xorEncrypted = this.crypto().xor1Encrypted("Hast-la$Vista=");
    assertTrue(this.crypto().equalsUTF8("{SSHA512}7pLyRczt1l4wim1jYvW/CSqm3XmuqjobGU3NS7+YQWiK5qjwrP7i7v3lOvBIyNoTxPjnU4/D0Y4+BTMH8VhSMSOzuU3gKBNVxdc+newf0IHXkCQe", xorEncrypted));
  }

  // Changed for PFL-58 : Comparing secrets must not return true if both are identical hashes
  // not assertTrue() anymore
  @Test
  public void test_equalsUTF8_both_hashed()
  {
    String b64 = Base64Converter.encode("{SHA256}rKLSV7WM5ycov0FiH6qEbKSYRNehAEz+T2c58IFNAN0=");
    assertFalse(this.crypto().equalsUTF8(b64, "{SHA256}rKLSV7WM5ycov0FiH6qEbKSYRNehAEz+T2c58IFNAN0="));
  }

  @Test
  public void test_equals_both_b64_encoded()
  {
    String str1 = Base64Converter.encode("318t38basv");
    String str2 = Base64Converter.encode("318t38basv");
    assertTrue(this.crypto().equals(str1, str2));
  }

  @Test
  public void test_equals_both_plain()
  {
    assertTrue(this.crypto().equals("Columbia", "Columbia"));
  }

  @Test
  public void test_equals_with_xor1()
  {
    String string = "Another one bites the dust";
    String str1 = this.crypto().base64HashedUTF8(string, CryptoConstants.SHA_2_384_NAME);
    String str2 = this.crypto().defaultEncrypted(string);
    assertTrue(this.crypto().equals(str1, str2, CheckedCharsetName.UTF_8));
  }

  @Test
  public void test_sshaEncrypted_1() throws Exception
  {
    String str = "MorningStar 2000";
    String ssha;

    ssha = this.crypto().sshaEncrypted(str);
    assertTrue(this.crypto().equals(str, ssha));
  }

  @Test
  public void test_sshaEncrypted_2() throws Exception
  {
    String str = "GridBag";
    byte[] salt = "abcdefg".getBytes();
    String ssha;

    ssha = this.crypto().sshaEncrypted(str, salt);
    assertTrue(this.crypto().equals(str, ssha));
  }

  @Test
  public void test_sshaEncrypted_3() throws Exception
  {
    String str = "Rainbow";
    byte[] salt = "warrior".getBytes();
    String hash1;
    String hash2;

    hash1 = this.crypto().sshaEncrypted(str, salt);
    Thread.currentThread().sleep(100);
    hash2 = this.crypto().sshaEncrypted(str, salt);
    assertTrue(this.crypto().equals(str, hash1));
    assertTrue(this.crypto().equals(str, hash2));
    assertEquals(hash1, hash2);
  }

  @Test
  public void test_sshaEncrypted_4() throws Exception
  {
    String str = "Rainbow";
    String hash1;
    String hash2;

    hash1 = this.crypto().sshaEncrypted(str);
    hash2 = this.crypto().sshaEncrypted(str);
    assertTrue(this.crypto().equals(str, hash1));
    assertTrue(this.crypto().equals(str, hash2));
    assertTrue(!hash1.equals(hash2));
  }

  @Test
  public void test_sshaEncrypted_null() throws Exception
  {
    assertNull(this.crypto().sshaEncrypted(null));
    assertNull(this.crypto().sshaEncrypted(null, null));
  }

  @Test
  public void test_shaEncrypted_1() throws Exception
  {
    String str = "Bandit";
    String sha;

    sha = this.crypto().shaEncrypted(str);
    assertTrue(this.crypto().equals(str, sha));
    assertEquals("{SHA}EdArUiAgkT6xnucV1+ni201KpKw=", sha);
  }

  @Test
  public void test_shaEncrypted_null() throws Exception
  {
    assertNull(this.crypto().shaEncrypted(null));
  }

  @Test
  public void test_computeHash_invalid_algorithm()
  {
    try
    {
      this.crypto().computeHash("AloneIntheDark", "ASCII", "INVALID");
      fail("Expected CryptoRuntimeException");
    }
    catch (CryptoRuntimeException ex)
    {
      assertTrue(ex.getCause() instanceof NoSuchAlgorithmException);
    }
  }

  @Test
  public void test_computeHash_invalid_char_encoding()
  {
    try
    {
      this.crypto().computeHash("AloneIntheDark", "UNKNOWN", CryptoConstants.ALGORITHM_NAME_SHA_2_256);
      fail("Expected CryptoRuntimeException");
    }
    catch (CryptoRuntimeException ex)
    {
      assertTrue(ex.getCause() instanceof UnsupportedEncodingException);
    }
  }

  @Test
  public void test_ISO_8859_UTF_8_char_encoding_comparison()
  {
    String text;
    byte[] hash1;
    byte[] hash2;
    String hashStr1;
    String hashStr2;

    // Just ACSII characters - both should be the same
    text = "Last Man Standing";
    hash1 = this.crypto().computeHash(CheckedCharsetName.ISO_8859_1.getBytes(text), CheckedHashAlgorithm.SHA_2_256);
    hash2 = this.crypto().computeHash(CheckedCharsetName.UTF_8.getBytes(text), CheckedHashAlgorithm.SHA_2_256);
    assertTrue(Arrays.equals(hash1, hash2));

    hashStr1 = this.crypto().hashed(CheckedCharsetName.ISO_8859_1.getBytes(text), HashMechanism.SHA_2_384);
    hashStr2 = this.crypto().hashed(CheckedCharsetName.UTF_8.getBytes(text), HashMechanism.SHA_2_384);
    assertTrue(hashStr1.equals(hashStr2));
    assertTrue(this.crypto().equals(hashStr1, text, CheckedCharsetName.ISO_8859_1));
    assertTrue(this.crypto().equals(hashStr2, text, CheckedCharsetName.UTF_8));

    // Now special characters - the hashes should be different
    text = "Füße";
    hash1 = this.crypto().computeHash(CheckedCharsetName.ISO_8859_1.getBytes(text), CheckedHashAlgorithm.SHA_2_256);
    hash2 = this.crypto().computeHash(CheckedCharsetName.UTF_8.getBytes(text), CheckedHashAlgorithm.SHA_2_256);
    assertFalse(Arrays.equals(hash1, hash2));
    assertFalse(this.crypto().equals(hashStr1, hashStr2)); // Changed from assertTrue() due to issue PF-58

    hashStr1 = this.crypto().hashed(CheckedCharsetName.ISO_8859_1.getBytes(text), HashMechanism.SHA_2_384);
    hashStr2 = this.crypto().hashed(CheckedCharsetName.UTF_8.getBytes(text), HashMechanism.SHA_2_384);
    assertFalse(hashStr1.equals(hashStr2));
    assertTrue(this.crypto().equals(hashStr1, text, CheckedCharsetName.ISO_8859_1));
    assertTrue(this.crypto().equals(hashStr2, text, CheckedCharsetName.UTF_8));
  }

  @Test
  public void test_hashed_null()
  {
    assertNull(this.crypto().hashed(null, HashMechanism.SALTED_SHA_1));
    assertNull(this.crypto().hashed(new byte[] { 1, 2, 3 }, null));
  }

  @Test
  public void test_computeUTF8Hash_SHA_256()
  {
    byte[] expected = SU.hexToBytes("34d9c23391c05837f2c68da86ebe3edb6e76a4d2a6e62b86b8bd03251534342c");
    byte[] result = this.crypto().computeUTF8Hash("AloneIntheDark", CryptoConstants.ALGORITHM_NAME_SHA_2_256);
    assertTrue(Arrays.equals(expected, result));
  }

  @Test
  public void test_computeUTF8Hash_SHA_384()
  {
    byte[] expected = SU.hexToBytes("72d6b0dd528e883910deb08085a11afb27d096ec1f399c3c7bf4823cc42ad1e422c5a99c7f5a05d02ff957173e525a9d");
    byte[] result = this.crypto().computeUTF8Hash("MorningHasBroken", CryptoConstants.ALGORITHM_NAME_SHA_2_384);
    assertTrue(Arrays.equals(expected, result));
  }

  @Test
  public void test_computeUTF8Hash_SHA_512()
  {
    byte[] expected = SU.hexToBytes("bda6c8815a2b66390a76651bd9fe2dd416884bfb62a50a237d8ec0ca28629bef6bf252d9b4f08ca4ab857bf330bc476b24261f3c9f24b5c74661ca433a5e6ce9");
    byte[] result = this.crypto().computeUTF8Hash("The$quick%brown&fox!jumped?over@the/lazy#dog.", CryptoConstants.ALGORITHM_NAME_SHA_2_512);
    assertTrue(Arrays.equals(expected, result));
  }

  @Test
  public void test_computeUTF8Hash_MD5()
  {
    byte[] expected = SU.hexToBytes("d3581554d2532db1a9a398e02747a455");
    byte[] result = this.crypto().computeUTF8Hash("WalkingIntheShadowOfTheBlues", CryptoConstants.ALGORITHM_NAME_MD5);
    assertTrue(Arrays.equals(expected, result));
  }

  @Test
  public void test_hashedUTF8_SHA_1()
  {
    String expected = "{SHA}Wsu9zpfIEhHoNbnog8dXeFHUNdA=";

    assertEquals(expected, this.crypto().hashedUTF8("Sunshine of your Love", CryptoConstants.SHA_1_NAME));
  }

  @Test
  public void test_hashedUTF8_MD5()
  {
    String text = "Sunshine of your Love";
    String hash1;
    String hash2;

    hash1 = this.crypto().hashedUTF8(text, HashMechanism.MD5);
    hash2 = this.crypto().hashedUTF8(text, HashMechanism.MD5);
    assertEquals(hash1, hash2);
  }

  @Test
  public void test_hashedUTF8_salted_MD5()
  {
    String text = "Sunshine of your Love";
    String hash;

    hash = this.crypto().hashedUTF8(text, HashMechanism.SALTED_MD5);
    assertTrue(this.crypto().equals(text, hash));
  }

  @Test
  public void test_hashedUTF8_null()
  {
    assertNull(this.crypto().hashedUTF8(null, CryptoConstants.SHA_1_NAME));
    assertNull(this.crypto().hashedUTF8(null, HashMechanism.MD5));
    assertNull(this.crypto().hashedUTF8("test", (String)null));
    assertNull(this.crypto().hashedUTF8("test", (HashMechanism)null));
  }

  @Test
  public void test_xor1_null()
  {
    assertNull(this.crypto().xor1(null));
  }

  @Test
  public void test_getFingerprintOf_no_separator() throws CertificateException
  {
    X509Certificate cert;

    cert = SECU.readX509CertificateFromPKCS7(new File(UnitTestHelper.TESTDATA + "public-cert-008.pb7"));
    assertEquals(UnitTestHelper.CERT_FINGERPRINT_008, this.crypto().getFingerprintOf(cert));
  }

  @Test
  public void test_getFingerprintOf_with_separator() throws CertificateException
  {
    X509Certificate cert;

    cert = SECU.readX509CertificateFromPKCS7(new File(UnitTestHelper.TESTDATA + "public-cert-004.pb7"));
    assertEquals(UnitTestHelper.CERT_FINGERPRINT_004, this.crypto().getFingerprintOf(cert, ":"));
  }

  // =========================================================================
  // HELPER METHODS
  // =========================================================================
  protected CryptoUtil crypto()
  {
    return CryptoUtil.current();
  }
}
