// ===========================================================================
// CONTENT  : TEST CLASS {ClassName}
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 22/06/2008
// HISTORY  :
//  22/06/2008  mdu  CREATED
//
// Copyright (c) 2008, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.security.crypto;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.junit.Assert.*;

import org.junit.Test;
import org.pfsw.text.CheckedCharsetName;
import org.pfsw.util.Base64Converter;

/**
 * Test class for corresponding business class.
 *
 * @author M.Duchrow
 * @version 1.0
 */
public class HashedStringTest
{
  @Test
  public void test_isEqual_sha_1()
  {
    HashedString pw;

    pw = new HashedString("{SHA}BKHwzdC4pYFd6ED5q1Xoko26BfE=");
    assertFalse(pw.isEqual("candle$Light".getBytes()));
    assertTrue(pw.isEqual("candle$L1ght".getBytes()));
  }

  @Test
  public void test_isEqual_ssha_1()
  {
    HashedString pw;

    pw = new HashedString("{SSHA}D650r/8U5balgMf7Kb95kGbYCJkyMTUyNjExNzcwNzU1Mw==");
    assertFalse(pw.isEqual("615bReak&Down"));
    assertTrue(pw.isEqual("635bReak&Down"));
  }

  @Test
  public void test_isEqual_ssha_2()
  {
    HashedString pw;

    pw = new HashedString("{SSHA}4OQd0q0mO9ETEN1uCXocG6f8etcyMTUyNjEyMDMwMTM1NQ==");
    assertFalse(pw.isEqual("615bReak&Down".getBytes()));
    assertTrue(pw.isEqual("635bReak&Down".getBytes()));
  }

  @Test
  public void test_isEqual_plain_1()
  {
    HashedString pw;

    pw = new HashedString("rainbowEyes");
    assertFalse(pw.isEqual("RainbowEyes".getBytes()));
    assertTrue(pw.isEqual("rainbowEyes".getBytes()));
  }

  @Test
  public void test_isEqual_plain_byte_constructor()
  {
    HashedString pw;

    pw = new HashedString("rainbowEyes".getBytes(), (String)null);
    assertFalse(pw.isEqual("RainbowEyes".getBytes()));
    assertTrue(pw.isEqual("rainbowEyes".getBytes()));
  }

  @Test
  public void test_asString_plain_text()
  {
    HashedString pw;
    String value = "Ofenrohr";

    pw = new HashedString(value);
    assertEquals(value, pw.asString());
  }

  @Test
  public void test_asString_SSHA_from_string_constructor()
  {
    HashedString pw;
    String value = "{SSHA}4OQd0q0mO9ETEN1uCXocG6f8etcyMTUyNjEyMDMwMTM1NQ==";

    pw = new HashedString(value);
    assertEquals(value, pw.asString());
  }

  @Test
  public void test_asString_SSHA256_from_bytes_constructor()
  {
    HashedString pw;
    String value = "{SHA256}s5BhQEdRPQRUX7EWdR1pK1qKKJe4kwYL2aV97CRsZst1ktkbiBAhQebfNmn+GzwmkFSGgg==";

    pw = new HashedString(value);
    assertEquals(value, pw.asString());
  }

  @Test
  public void test_isEqualUTF8_SSHA256()
  {
    HashedString pw;
    String text = "Samba Pa Ti";

    pw = new HashedString(CheckedCharsetName.UTF_8.getBytes(text), CryptoUtil.current().generateSalt20(), CryptoConstants.SSHA_2_256_PREFIX);
    assertTrue(pw.isEqualUTF8(text));
  }

  @Test
  public void test_isEqual_SSHA256_encoded()
  {
    HashedString pw;
    String text = "Samba Pa Ti";

    pw = new HashedString("{SSHA256}U/0x8+H64m9VDc3EBuhTFujoGIwQyn4v7pkOBQ6ApVPwbCREj8qsqRBgN0i4QWo8veP6nQ==");
    assertTrue(pw.isEqual(text));
  }

  @Test
  public void test_isEqualUTF8_SSHA384()
  {
    HashedString pw;
    String text = "Blues Brothers are in town";

    pw = new HashedString(CheckedCharsetName.UTF_8.getBytes(text), CryptoConstants.SSHA_2_384_NAME);
    assertTrue(pw.isEqualUTF8(text));
  }

  @Test
  public void test_isEqualUTF8_SSHA512()
  {
    HashedString pw;
    String text = "Alan Parsons Project";

    pw = new HashedString(CheckedCharsetName.UTF_8.getBytes(text), CryptoUtil.current().generateSalt20(), CryptoConstants.SSHA_2_512_NAME);
    assertTrue(pw.isEqualUTF8(text));
  }

  @Test
  public void test_isEqual_SSHA512_encoded()
  {
    HashedString hs;
    String text = "Alan Parsons Project";

    hs = new HashedString("{SSHA512}NqK8XtL0sCXnM47Y5GLyENmIIMw6AMKNt+qVgZvMHXKedvBoiGpZkbxIKPMdntDaL5dW0OTFiZLyyxFV8BOtXzQqKzuS1JaginZyVouzKHis9Akx");
    assertTrue(hs.isEqual(text));
  }

  @Test
  public void test_isSalted_SSHA256()
  {
    HashedString hs;
    byte[] bytes = { 65, 66, 67 };

    hs = new HashedString(bytes, HashMechanism.SALTED_SHA_2_256);
    assertTrue(hs.isSalted());
  }

  @Test
  public void test_isSalted_no_mechanism()
  {
    HashedString hs;

    hs = new HashedString("text");
    assertFalse(hs.isSalted());
  }

  @Test
  public void test_isHashed_SHA384()
  {
    HashedString hs;
    byte[] bytes = { 65, 66, 67 };

    hs = new HashedString(bytes, HashMechanism.SHA_2_384);
    assertTrue(hs.isHashed());
  }

  @Test
  public void test_isHashed_no_mechanism()
  {
    HashedString hs;

    hs = new HashedString("text");
    assertFalse(hs.isHashed());
  }

  @Test
  public void test_getHashValue_SHA256()
  {
    String expected = "zuv3MN1gR7h0cmM2yoyQIdf/hQajWFvB5SICPicqr6A=";
    String result;
    HashedString hashedString = new HashedString("Westminster".getBytes(), HashMechanism.SHA_2_256);

    result = Base64Converter.encodeToString(hashedString.getHashValue());
    assertEquals(expected, result);
  }

  @Test
  public void test_getSaltValue_SSHA384()
  {
    byte[] salt = "ABC".getBytes();
    HashedString hashedString = new HashedString("Westminster".getBytes(), salt, HashMechanism.SALTED_SHA_2_384);

    assertEquals(salt, hashedString.getSaltValue());
  }
}
