package org.pfsw.security.oauth.pkce;

import static org.junit.Assert.*;
import static org.pfsw.security.testhelper.UnitTestHelper.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.pfsw.bif.ByteSequenceGenerator;
import org.pfsw.security.Base64UrlEncodedRandomStringGenerator;
import org.pfsw.security.testhelper.AlwaysNewS256PKCEGenerator;
import org.pfsw.security.testhelper.PrimitiveRandomStringGenerator;
import org.pfsw.text.StringGenerator;

public class S256PKCEGeneratorTest
{
  @Test
  public void test_generate()
  {
    assertS256Key(S256PKCEGenerator.instance().generate());
  }

  @Test
  public void test_newProofKey()
  {
    assertS256Key(S256PKCEGenerator.newProofKey());
  }

  @Test
  public void test_generate__different_keys_with_always_new_generator()
  {
    assertDifferentKeys(new AlwaysNewS256PKCEGenerator());
  }

  @Test
  public void test_generate__different_keys_default_string_generator()
  {
    S256PKCEGenerator pkceGenerator = S256PKCEGenerator.create();
    assertDifferentKeys(pkceGenerator);
  }

  @Test
  public void test_generate__different_keys_other_string_generator()
  {
    S256PKCEGenerator pkceGenerator = S256PKCEGenerator.create(new PrimitiveRandomStringGenerator());
    assertDifferentKeys(pkceGenerator);
  }

  @Test
  public void test_generate__spec_example()
  {
    ByteSequenceGenerator byteSequenceGenerator;
    StringGenerator stringGenerator;
    S256PKCEGenerator pkceGenerator;
    ProofKeyForCodeExchange proofKey;
    
    byteSequenceGenerator = new ByteSequenceGenerator()
    {
      @SuppressWarnings("synthetic-access")
      private byte[] byteSequence = { 116, 24, asByte(223), asByte(180), asByte(151), asByte(153), asByte(224), 37, 79, asByte(250), 96, 125, asByte(216),
          asByte(173), asByte(187), asByte(186), 22, asByte(212), 37, 77, 105, asByte(214), asByte(191), asByte(240), 91, 88, 5, 88, 83, asByte(132),
          asByte(141), 121 };
      
      @Override
      public byte[] generateByteSequence()
      {
        return byteSequence;
      }
    };
    
    stringGenerator = Base64UrlEncodedRandomStringGenerator.create(byteSequenceGenerator);

    pkceGenerator = S256PKCEGenerator.create(stringGenerator);
    proofKey = pkceGenerator.generate();
    assertEquals("dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk", proofKey.getCodeVerifier());
    assertEquals("E9Melhoa2OwvFrEMTJguCHaoeK1t8URWbuGJSstw-cM", proofKey.getCodeChallenge());
  }

  // ============== helper methods ======================================

  private static byte asByte(int value)
  {
    return (byte)value;
  }

  private void assertDifferentKeys(PKCEGenerator pkceGenerator)
  {
    int numberOfTests = 5000;
    List<ProofKeyForCodeExchange> proofKeys = new ArrayList<ProofKeyForCodeExchange>();
    ProofKeyForCodeExchange proofKey;

    for (int i = 0; i < numberOfTests; i++)
    {
      proofKey = pkceGenerator.generate();
      assertS256Key(proofKey);
      for (ProofKeyForCodeExchange previousKey : proofKeys)
      {
        assertFalse(proofKey.getCodeVerifier().equals(previousKey.getCodeVerifier()));
      }
      proofKeys.add(proofKey);
    }

  }

  private void assertS256Key(ProofKeyForCodeExchange proofKey)
  {
    assertFalse(proofKey.getCodeVerifier().equals(proofKey.getCodeChallenge()));
    assertTrue(CodeChallengeMethod.S256 == proofKey.getCodeChallengeMethod());
    assertProofKeyCodeVerifier(proofKey.getCodeVerifier());
  }
}
