// ===========================================================================
// CONTENT  : CLASS MultiX509TrustManagerTest
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 02/08/2021
// HISTORY  :
//  02/08/2021  mdu  CREATED
//
// Copyright (c) 2021, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.security.x509;

import static org.junit.Assert.*;
import static org.pfsw.security.SecurityUtil.*;
import static org.pfsw.security.testhelper.UnitTestHelper.*;

import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import org.junit.Test;

public class MultiX509TrustManagerTest
{
  @Test
  public void test_checkServerTrusted_Bielefeld()
  {
    CertificateException exception = null;
    // The following certificate is signed by root CA "T-TeleSec GlobalRoot Class 2" which is valid until "Sat, 01 Oct 2033 23:59:59 GMT"
    // This root cert is available in JDK 1.6.0_41 (and later).
    X509Certificate[] certChain = SECU.readX509CertificateChainFromPEM(testDataFile("www-bielefeld-de-chain.pem"));

    //@formatter:off
    MultiX509TrustManager trustManager = X509TrustManagerBuilder.create()
      .addDefaultTrustStore()
      .build();
    //@formatter:on

    try
    {
      trustManager.checkServerTrusted(certChain, "RSA");
    }
    catch (CertificateException e)
    {
      exception = e;
    }
    assertNull(exception);
  }

  @Test
  public void test_checkServerTrusted_self_signed()
  {
    CertificateException exception = null;
    // The following certificate is signed by root CA "DigiCert Global Root CA" which is valid until "Mon, 10 Nov 2031 00:00:00 GMT"
    // This root cert is available in JDK 1.6.0_41 (and later).
    X509Certificate[] certChain1 = SECU.readX509CertificateChainFromPEM(testDataFile("austintexas-org-chain.pem"));
    // This is a self-signed certificate that cannot be verified by the default truststore! It is valid until August 2071
    X509Certificate[] certChain2 = SECU.readX509CertificateChainFromPEM(testDataFile("testserver1-testcompany-test-chain.pem"));

    //@formatter:off
    MultiX509TrustManager trustManager = X509TrustManagerBuilder.create()
        .addDefaultTrustStore()
        .addTrustStore(TESTDATA + "truststore1.jks", "password")
        .build();
    //@formatter:on

    try
    {
      trustManager.checkServerTrusted(certChain1, "RSA");
      trustManager.checkServerTrusted(certChain2, "RSA");
    }
    catch (CertificateException e)
    {
      exception = e;
    }
    assertNull(exception);
  }

  @Test(expected = CertificateException.class)
  public void test_checkServerTrusted_self_signed_not_valid() throws CertificateException
  {
    X509Certificate[] certChain = SECU.readX509CertificateChainFromPEM(testDataFile("testserver1-testcompany-test-chain.pem"));

    //@formatter:off
    MultiX509TrustManager trustManager = X509TrustManagerBuilder.create()
        .addDefaultTrustStore()
        .build();
    //@formatter:on

    trustManager.checkServerTrusted(certChain, "RSA");
  }

  @Test
  public void test_getAcceptedIssuers()
  {
    //@formatter:off
    MultiX509TrustManager trustManager = X509TrustManagerBuilder.create()
        .addDefaultTrustStore()
        .addTrustStore(TESTDATA + "truststore1.jks", "password")
        .build();
    //@formatter:on
    
    X509Certificate[] acceptedIssuers = trustManager.getAcceptedIssuers();
    assertTrue(acceptedIssuers.length > 20);
  }

  @Test
  public void test_checkClientTrusted_self_signed()
  {
    CertificateException exception = null;

    // This is a self-signed certificate that cannot be verified by the default truststore! It is valid until August 2071
    X509Certificate[] certChain = SECU.readX509CertificateChainFromPEM(testDataFile("client1.pem"));

    //@formatter:off
    MultiX509TrustManager trustManager = X509TrustManagerBuilder.create()
        .addDefaultTrustStore()
        .addTrustStore(TESTDATA + "truststore1.jks", "password")
        .build();
    //@formatter:on

    try
    {
      trustManager.checkClientTrusted(certChain, "RSA");
    }
    catch (CertificateException e)
    {
      exception = e;
    }
    assertNull(exception);
  }
}

